@@ -1,12 +1,14 @@
-from syrenka.classdiagram import SyrenkaClassDiagram, SyrenkaClassDiagramConfig
-from syrenka.base import ThemeNames
-from syrenka.lang.python import PythonModuleAnalysis
+"""Example SyrenkaClassDiagram."""
 
 # from io import StringIO
 import sys
 
+from syrenka.base import ThemeNames
+from syrenka.classdiagram import SyrenkaClassDiagram, SyrenkaClassDiagramConfig
+from syrenka.lang.python import PythonModuleAnalysis
+
 class_diagram = SyrenkaClassDiagram(
-    "syrenka class diagram", SyrenkaClassDiagramConfig().theme(ThemeNames.neutral)
+    "syrenka class diagram", SyrenkaClassDiagramConfig().theme(ThemeNames.NEUTRAL)
 )
 class_diagram.add_classes(
     PythonModuleAnalysis.classes_in_module(module_name="syrenka", nested=True)
