@@ -1,14 +1,15 @@
-from syrenka.classdiagram import SyrenkaClassDiagram, SyrenkaClassDiagramConfig
-from syrenka.base import ThemeNames
-from syrenka.lang.python import PythonModuleAnalysis
-
-from pathlib import Path
+"""Example SyrenkaClassDiagram with ast backend."""
 
 # from io import StringIO
 import sys
+from pathlib import Path
+
+from syrenka.base import ThemeNames
+from syrenka.classdiagram import SyrenkaClassDiagram, SyrenkaClassDiagramConfig
+from syrenka.lang.python import PythonModuleAnalysis
 
 class_diagram = SyrenkaClassDiagram(
-    "syrenka class diagram", SyrenkaClassDiagramConfig().theme(ThemeNames.neutral)
+    "syrenka class diagram", SyrenkaClassDiagramConfig().theme(ThemeNames.NEUTRAL)
 )
 class_diagram.add_classes(
     PythonModuleAnalysis.classes_in_path(
