@@ -1,5 +1,10 @@
+"""Example SyrenkaFlowchart usage."""
+
+from io import StringIO
+from pathlib import Path
+
 import syrenka.flowchart as sf
-import sys
+from syrenka.generate import generate_diagram_image
 
 # flowchart TB
 #     c1-->a2
@@ -17,11 +22,9 @@ import sys
 #     two --> c2
 # from https://mermaid.js.org/syntax/flowchart.html
 
-# TODO: Edges
-
 flowchart = sf.SyrenkaFlowchart(
     "",
-    sf.FlowchartDirection.TopToBottom,
+    sf.FlowchartDirection.TOP_TO_BOTTOM,
     nodes=[
         sf.Subgraph(
             "one",
@@ -32,6 +35,7 @@ flowchart = sf.SyrenkaFlowchart(
         ),
         sf.Subgraph(
             "two",
+            direction=sf.FlowchartDirection.LEFT_TO_RIGHT,
             nodes=[
                 sf.Node("b1"),
                 sf.Node("b2"),
@@ -39,6 +43,7 @@ flowchart = sf.SyrenkaFlowchart(
         ),
         sf.Subgraph(
             "three",
+            direction=sf.FlowchartDirection.BOTTOM_TO_TOP,
             nodes=[
                 sf.Node("c1"),
                 sf.Node("c2"),
@@ -55,4 +60,11 @@ flowchart.connect_by_id("one", "two").connect_by_id("three", "two").connect_by_i
 
 # beware, it looks like in mermaid order of the edge changes how its rendered
 # if i declare edge c1 --> a2 before and after subgraphs, it gets drawn totally different
-flowchart.to_code(file=sys.stdout)
+
+
+out = StringIO()
+flowchart.to_code(file=out)
+
+print(out.getvalue())
+
+generate_diagram_image(out.getvalue(), Path("out.svg"), overwrite=True)
