@@ -1,29 +1,26 @@
+import ast
+import importlib
+import logging
+import sys
 from collections.abc import Iterable
-from inspect import isclass, ismodule
+from dataclasses import dataclass
+from enum import Enum
+from inspect import getfullargspec, isbuiltin, isclass, ismethoddescriptor, ismodule
 from pathlib import Path
 from types import ModuleType
-import importlib
-import ast
-from enum import Enum
-from dataclasses import dataclass
-
-import sys
-from inspect import getfullargspec, isbuiltin, ismethoddescriptor
+from typing import Union
 
 from syrenka.base import dunder_name
-
 from syrenka.lang.base import (
     LangAccess,
     LangAnalysis,
     LangAttr,
     LangClass,
-    LangVar,
     LangFunction,
+    LangVar,
     register_lang_analysis,
 )
 
-import logging
-
 logger = logging.getLogger(__name__)
 
 
@@ -31,6 +28,67 @@ SKIP_BASES = True
 SKIP_BASES_LIST = ["object", "ABC"]
 
 
+def ast_to_text(node) -> str:
+    text = None
+    if node is None:
+        text = ""
+    elif isinstance(node, ast.Attribute):
+        text = f"{ast_to_text(node.value)}.{node.attr}"
+    elif isinstance(node, ast.Call):
+        text = f"{ast_to_text(node.func)}({ast_to_text(node.args)})"
+    elif isinstance(node, Iterable):
+        name = []
+        for arg in node:
+            name.append(ast_to_text(arg))
+        text = ", ".join(name)
+    elif isinstance(node, ast.Name):
+        text = node.id
+    elif isinstance(node, ast.Constant):
+        if isinstance(node.value, str):
+            text = "'" + node.value + "'"
+        else:
+            text = str(node.value)
+    elif isinstance(node, ast.Tuple):
+        text = "(" + ast_to_text(node.elts) + ")"
+    elif isinstance(node, ast.List):
+        name = []
+        for elt in node.elts:
+            name.append(ast_to_text(elt))
+        text = "[" + ast_to_text(node.elts) + "]"
+    elif isinstance(node, ast.Dict):
+        name = []
+        for i, key in enumerate(node.keys):
+            name.append(f"{ast_to_text(key)}: {ast_to_text(node.values[i])}")
+        text = "{" + ", ".join(name) + "}"
+    elif isinstance(node, ast.Subscript):
+        if node.slice:
+            # The slice can be other types:
+            # Tuple [A, B]
+            # Slice [1:2]
+            if isinstance(node.slice, ast.Tuple):
+                # for tuple we get (), but for case in subscript, we want brackets
+                slice_txt = ast_to_text(node.slice.elts)
+            else:
+                slice_txt = ast_to_text(node.slice)
+        else:
+            slice_txt = ""
+        text = ast_to_text(node.value) + "[" + slice_txt + "]"
+    elif isinstance(node, ast.Slice):
+        text = ast_to_text(node.lower) + ":" + ast_to_text(node.upper)
+    elif isinstance(node, ast.IfExp):
+        text = (
+            ast_to_text(node.body)
+            + " if "
+            + ast_to_text(node.test)
+            + " else "
+            + ast_to_text(node.orelse)
+        )
+    else:
+        raise NotImplementedError(f"Unsupported node to text: {node}")
+
+    return text
+
+
 @dataclass(frozen=True)
 class PythonAstModuleParams:
     ast_module: ast.Module
@@ -43,7 +101,7 @@ class PythonAstClassParams:
     ast_class: ast.ClassDef
     filepath: Path
     root: Path
-    module_name: str | None
+    module_name: Union[str, None] = None
 
 
 class PythonAstClass(LangClass):
@@ -58,7 +116,7 @@ class PythonAstClass(LangClass):
         self._namespace = None
 
         # TODO: Should be done properly as some PythonAstModule, but it works, refactor later
-        if type(self.cls) is ast.ClassDef:
+        if isinstance(self.cls, ast.ClassDef):
             self._name = self.cls.name
         else:
             self._name = "_globals_"
@@ -77,15 +135,25 @@ class PythonAstClass(LangClass):
         if hasattr(self.cls, "decorator_list") and self.cls.decorator_list:
             for decorator in self.cls.decorator_list:
                 # might be dataclass
-                if type(decorator) is ast.Call:
-                    if decorator.func.id == "dataclass":
+                if isinstance(decorator, ast.Call):
+                    decorator_name = None
+
+                    # for @dataclasses.dataclass we will get Attribute dataclass with Name dataclasses
+                    # for @dataclass we will get ast.Name in func
+
+                    if isinstance(decorator.func, ast.Attribute):
+                        decorator_name = decorator.func.attr
+                    elif isinstance(decorator.func, ast.Name):
+                        decorator_name = decorator.func.id
+
+                    if decorator_name and decorator_name == "dataclass":
                         is_dataclass = True
-                elif type(decorator) is ast.Name:
+                elif isinstance(decorator, ast.Name):
                     if decorator.id == "dataclass":
                         is_dataclass = True
 
         for ast_node in self.cls.body:
-            if type(ast_node) is ast.Assign:
+            if isinstance(ast_node, ast.Assign):
                 # if type(ast_node.value) not in [ast.Constant, ast.Name, ast.Call]:
                 #     logger.debug(
                 #         f"ast.Asign - discarded ({type(ast_node.value)}) {ast_node.value = }"
@@ -93,9 +161,9 @@ class PythonAstClass(LangClass):
                 #     continue
 
                 for target in ast_node.targets:
-                    if type(target) is not ast.Name:
+                    if not isinstance(target, ast.Name):
                         logger.debug(
-                            f"ast.Assign - discarded ({type(target)}) {target = }"
+                            "ast.Assign target discard - %s, %s", type(target), target
                         )
                         continue
 
@@ -107,7 +175,7 @@ class PythonAstClass(LangClass):
                         ),
                     )
 
-            if is_dataclass and type(ast_node) is ast.AnnAssign:
+            if is_dataclass and isinstance(ast_node, ast.AnnAssign):
                 # eg. name: str
                 # ast_node.annotation # ast.Name
                 attributes.append(
@@ -120,37 +188,45 @@ class PythonAstClass(LangClass):
                     )
                 )
 
-            if type(ast_node) is not ast.FunctionDef:
+            if not isinstance(ast_node, ast.FunctionDef):
                 # print(ast_node)
                 continue
 
             args_list = []
             for ast_arg in ast_node.args.args:
                 if ast_arg.annotation:
-                    if type(ast_arg.annotation) is ast.BinOp:
+                    if isinstance(ast_arg.annotation, ast.BinOp):
                         # theme_name: ThemeNames | str
                         # TODO
                         args_list.append(LangVar(ast_arg.arg))
                         continue
 
-                    if type(ast_arg.annotation) is ast.Subscript:
+                    if isinstance(ast_arg.annotation, ast.Subscript):
                         # text: Union[str, None] = None,
                         # TODO
                         args_list.append(LangVar(ast_arg.arg))
                         continue
 
-                    if type(ast_arg.annotation) is ast.Name:
-                        args_list.append(LangVar(ast_arg.arg, ast_arg.annotation.id))
+                    if isinstance(ast_arg.annotation, ast.Name):
+                        args_list.append(
+                            LangVar(ast_arg.arg, ast_to_text(ast_arg.annotation))
+                        )
                         continue
 
-                    if type(ast_arg.annotation) is ast.Attribute:
-                        typee = (
-                            ast_arg.annotation.value.id + "." + ast_arg.annotation.attr
-                        )
+                    if isinstance(ast_arg.annotation, ast.Attribute):
+                        typee = ast_to_text(ast_arg.annotation)
                         args_list.append(LangVar(ast_arg.arg, typee))
                         continue
 
-                    raise Exception("TODO not handled")
+                    if isinstance(ast_arg.annotation, ast.Constant):
+                        args_list.append(
+                            LangVar(ast_arg.arg, ast_to_text(ast_arg.annotation))
+                        )
+                        continue
+
+                    raise NotImplementedError(
+                        f"TODO - {ast_arg.annotation} not handled"
+                    )
 
                 lv = LangVar(ast_arg.arg)
 
@@ -170,7 +246,13 @@ class PythonAstClass(LangClass):
         self.info["functions"] = functions
         self.info["attributes"] = attributes
 
-        is_enum = any(map(lambda x: "enum" in x.lower(), self.parents()))
+        parents = []
+        if hasattr(self.cls, "bases"):
+            for base in self.cls.bases:
+                parents.append(ast_to_text(base))
+        self.info["parents"] = parents
+
+        is_enum = any(map(lambda x: "enum" in x.lower(), self.info["parents"]))
 
         if is_enum:
             self.info["enum"] = attribute_assign
@@ -220,12 +302,8 @@ class PythonAstClass(LangClass):
         return self.info["attributes"]
 
     def parents(self) -> Iterable[str]:
-        parents = []
-        if hasattr(self.cls, "bases"):
-            for base in self.cls.bases:
-                parents.append(base.id)
-
-        return parents
+        self._parse()
+        return self.info["parents"]
 
 
 @dataclass(frozen=True)
@@ -313,7 +391,7 @@ class PythonClass(LangClass):
                         PythonModuleAnalysis.get_access_from_name(x),
                     )
                 )
-            elif type(attr) is self.cls:
+            elif isinstance(attr, self.cls):
                 # enum values are instances of this enum
                 enum_values.append(
                     LangAttr(
@@ -369,11 +447,11 @@ class PythonModuleAnalysis(LangAnalysis):
         return type(obj) in [PythonAstClassParams, PythonClassParams]
 
     @staticmethod
-    def create_lang_class(obj) -> LangClass:
-        if type(obj) is PythonAstClassParams:
+    def create_lang_class(obj) -> Union[LangClass, None]:
+        if isinstance(obj, PythonAstClassParams):
             return PythonAstClass(obj)
 
-        if type(obj) is PythonClassParams:
+        if isinstance(obj, PythonClassParams):
             return PythonClass(obj)
 
         return None
@@ -392,7 +470,7 @@ class PythonModuleAnalysis(LangAnalysis):
         module_names = []
         stash = [module]
 
-        while len(stash):
+        while stash:
             m = stash.pop()
             if m.__name__ in module_names:
                 # circular?
@@ -445,11 +523,11 @@ class PythonModuleAnalysis(LangAnalysis):
     @staticmethod
     def classes_in_path(
         path: Path,
-        module_name: str | None = None,
+        module_name: Union[str, None] = None,
         recursive: bool = True,
         detect_project_dir: bool = True,
-        exclude: Iterable[str] | None = None,
-        only: Iterable[str] | None = None,
+        exclude: Union[Iterable[str], None] = None,
+        only: Union[Iterable[str], None] = None,
         globals_as_class: bool = False,
     ) -> Iterable[PythonAstClassParams]:
         root = path
@@ -479,7 +557,7 @@ class PythonModuleAnalysis(LangAnalysis):
                 if exclude and any(map(lambda x: rel_path.startswith(x), exclude)):
                     append = False
 
-                logger.debug(f"{rel_path=}, {append=}")
+                logger.debug("rel_path=%s, append=%s", rel_path, append)
                 if append:
                     ast_modules.append(
                         PythonAstModuleParams(
@@ -498,15 +576,15 @@ class PythonModuleAnalysis(LangAnalysis):
     def get_classes_from_ast(
         ast_modules: Iterable[PythonAstModuleParams],
         root: Path,
-        module_name: str | None,
+        module_name: Union[str, None],
     ) -> Iterable[PythonAstClassParams]:
         class_params = []
         # this is shallow, we dont take into account classes in classes
         for params in ast_modules:
             found_non_class = False
             for ast_node in params.ast_module.body:
-                logger.debug(f"{params.filepath.as_posix()}: {ast_node}")
-                if type(ast_node) is ast.ClassDef:
+                logger.debug("%s: %s", params.filepath.as_posix(), ast_node)
+                if isinstance(ast_node, ast.ClassDef):
                     class_params.append(
                         PythonAstClassParams(
                             ast_class=ast_node,
@@ -520,10 +598,9 @@ class PythonModuleAnalysis(LangAnalysis):
                         # not in [ast.Import, ast.ImportFrom]:
                         found_non_class = True
                     # open: only function defs, do we want assigns
-                    pass
             if params.globals_as_class and found_non_class:
                 logger.debug(
-                    f"{params.filepath.as_posix()}: adding _globals_ pseudo-class"
+                    "%s: adding _globals_ pseudo-class", params.filepath.as_posix()
                 )
                 class_params.append(
                     PythonAstClassParams(
@@ -536,8 +613,8 @@ class PythonModuleAnalysis(LangAnalysis):
         return class_params
 
     @staticmethod
-    def get_ast(filename: Path | str):
-        if type(filename) is str:
+    def get_ast(filename: Union[Path, str]):
+        if isinstance(filename, str):
             filename = Path(filename)
 
         if not filename.exists():
@@ -545,58 +622,67 @@ class PythonModuleAnalysis(LangAnalysis):
 
         ast_module = PythonModuleAnalysis.ast_cache.get(filename, None)
         if ast_module is None:
-            with filename.open("r", encoding="utf-8") as f:
-                ast_module = ast.parse(f.read(), str(filename.name))
+            # open file as binary and pass it to ast, it can handle different encodings
+            # if we open it as regular "r" we will possibly get decode errors in case of some files
+            # as it will be open with SOME encoding
+            with filename.open("rb") as f:
+                try:
+                    ast_module = ast.parse(f.read(), str(filename.name))
+                except SyntaxError as ex:
+                    print(f"FAIL: File doesn't parse correctly: {ex}", file=sys.stderr)
+                    raise ex
             PythonModuleAnalysis.ast_cache[filename] = ast_module
 
         return ast_module
 
     @staticmethod
-    def get_ast_node(filename: Path | str, firstlineno, ast_type):
+    def get_ast_node(filename: Union[Path, str], firstlineno, ast_type):
         ast_module = PythonModuleAnalysis.get_ast(filename)
 
         ast_nodes = [ast_module]
         while ast_node := ast_nodes.pop():
-            if type(ast_node) is ast_type and ast_node.lineno == firstlineno:
+            if isinstance(ast_node, ast_type) and ast_node.lineno == firstlineno:
                 break
 
             for child in ast_node.body:
-                if child.lineno <= firstlineno and child.end_lineno >= firstlineno:
+                if child.lineno <= firstlineno <= child.end_lineno:
                     ast_nodes.append(child)
                     break
 
         return ast_node
 
     @staticmethod
-    def get_ast_function(filename: Path | str, firstlineno) -> ast.FunctionDef:
+    def get_ast_function(filename: Union[Path, str], firstlineno) -> ast.FunctionDef:
         return PythonModuleAnalysis.get_ast_node(filename, firstlineno, ast.FunctionDef)
 
     @staticmethod
     def get_access_from_name(name):
         if name[0] == "_":
             if not dunder_name(name):
-                return LangAccess.Private
+                return LangAccess.PRIVATE
 
-        return LangAccess.Public
+        return LangAccess.PUBLIC
 
     @staticmethod
     def get_assign_attributes(ast_function: ast.FunctionDef) -> Iterable[LangAttr]:
         attributes = {}
         for entry in ast_function.body:
-            if type(entry) is not ast.Assign:
+            if not isinstance(entry, ast.Assign):
                 continue
 
+            ast_attribute = None
             for target in entry.targets:
-                if type(target) is ast.Attribute:
+                if isinstance(target, ast.Attribute):
+                    ast_attribute = target
                     break
 
-            if type(target) is not ast.Attribute:
+            if ast_attribute is None:
                 continue
 
-            attributes[target.attr] = LangAttr(
-                name=target.attr,
+            attributes[ast_attribute.attr] = LangAttr(
+                name=ast_attribute.attr,
                 typee=None,
-                access=PythonModuleAnalysis.get_access_from_name(target.attr),
+                access=PythonModuleAnalysis.get_access_from_name(ast_attribute.attr),
             )
 
         return attributes.values()
